const apiutils = require("api-utils");
const collection = require("collection");
const errors = require("errors");
const storage = require("simple-storage").storage;
const windowItems = require("window-items");

const storageKey = "oldgui-toolbar-nav-bar-currentSet";

const EVENTS = {
  onClick: "command",
  onMouseover: "mouseover",
  onMouseout: "mouseout"
};

function Widget(options){
  options = apiutils.validateOptions(options, {
    label: {
      is: ["null", "undefined", "string"],
    },
    tooltip: {
      is: ["null", "undefined", "string"],
    },
    image: {
      is: ["string"],
    },
    onClick: {
      is: ["function", "array", "null", "undefined"],
    },
    onMouseover: {
      is: ["function", "array", "null", "undefined"],
    },
    onMouseout: {
      is: ["function", "array", "null", "undefined"],
    }
  });

  this.__defineGetter__("label", function() options.label);

  this.__defineGetter__("tooltip", function() options.tooltip || options.label);
  this.__defineSetter__("tooltip", function(text) {
    options.tooltip = text;
    manager.updateItem(this, "tooltip", text);
  });
  this.__defineGetter__("image", function() options.image);
  this.__defineSetter__("image", function(image) {
    options.image = image;
    manager.updateItem(this, "image", image);
  });
  for (let method in EVENTS) {
    collection.addCollectionProperty(this, method);
    if (options[method]){
      this[method].add(options[method]);
    }
  }
  this.toString = function() {
    return '[object Toolbar-Widget "' + options.label + '"]';
  };
}

exports.Widget = apiutils.publicConstructor(Widget);

function getPalette(win) {
  return win.document.getElementById("navigator-toolbox").palette;
}

function getItemId(item) {
  return "oldgui_toolbar_" + require("self").id + "_" + item.id;
}

function getIndex(parent, id) {
  var childNodes = parent.childNodes;
  for (var i = 0; i < childNodes.length; ++i) {
    var c = childNodes[i];
    if(c.id == id){ return i; }
  }
  return -1;
}

function getItemNode(win, item) {
  var id = getItemId(item);
  var x = win.document.getElementById(id);
  if(x){ return x; }
  var palette = getPalette(win);
  var index = getIndex(palette, id);
  if(index >= 0) {
    return palette.childNodes[index];
  }
  return null;
}

function addEventHandlers(node, item) {
  for (let [method, type] in Iterator(EVENTS)) {
    for(let listener in item[method]) {
      node.addEventListener(type, function(e){
        errors.catchAndLog(function(e) listener.apply(item, [e]))(e);
      }, false);
    }
  }
}

function ToolbarManager() {
  this.map = [];
  require("unload").ensure(this);
};

ToolbarManager.prototype = {
  restore: function(navBar, item){
    let currentSet = storage[storageKey];
    if(currentSet && currentSet.indexOf(getItemId(item)) >= 0){
      navBar.currentSet = currentSet;
    }
  },
  observe: function(navBar){
    if(this.map.some(function(e) e.target == navBar)){
      // already listening
      return;
    }
    let listener = function() {
       let currentSet = navBar.currentSet;
       // dirty fix
       currentSet = currentSet.replace(
         "urlbar-container,splitter,search-container",
         "urlbar-container,search-container");
       storage[storageKey] = currentSet;
    };
    this.map.push({ target: navBar, listener: listener });
    navBar.parentNode.addEventListener("aftercustomization", listener, false);
  },
  unload : function(){
    this.map.forEach(function(e){
      let toolbox = e.target.parentNode;
      let listener = e.listener;
      toolbox.removeEventListener("aftercustomization", listener, false);
    });
  }
};

var toolbarManager = new ToolbarManager();

var manager = windowItems.Manager({
  onTrack: function(win, item) {
    let node = win.document.createElement("toolbarbutton");
    if(item.id) node.setAttribute("id", getItemId(item));
    node.setAttribute("class", "toolbarbutton-1");
    if(item.tooltip) {
      node.setAttribute("label", item.tooltip);
      node.setAttribute("tooltiptext", item.tooltip);
    }
    if(item.image) {
      node.setAttribute("style", "list-style-image: url('" + item.image + "');");
    }
    addEventHandlers(node, item);
    getPalette(win).appendChild(node);

    let navBar = win.document.getElementById("nav-bar");
    toolbarManager.restore(navBar, item);
    toolbarManager.observe(navBar);
  },

  onUntrack: function(win, item) {
    let node = getItemNode(win, item);
    if(node) {
      node.parentNode.removeChild(node);
    }
  },

  onUpdate: function(win, item, property, value) {
    let node = getItemNode(win, item);
    if(node) {
      if(property == "tooltip"){
        if(value){
          node.setAttribute("label", value);
          node.setAttribute("tooltiptext", value);
        }else{
          node.removeAttribute("label");
          node.removeAttribute("tooltiptext");
        }
      }else if(property == "image"){
        node.setAttribute("style", "list-style-image: url('" + item.image + "');");
      }
    }
  }
});

exports.add = function(button) {
  manager.addItem(button);
};

exports.remove = function(button) {
  manager.removeItem(button);
};
